Module ELog

    Dim timeout As Double = 10
    Dim startTime As DateTime
    Dim logsToSearch As String() = {"application", "security", "system"}
    Dim FoundError As Boolean = False
    Dim FoundWarning As Boolean = False
    Dim period As Double = 1440
    Dim includes As New Generic.Dictionary(Of String, Generic.List(Of String))
    Dim excludes As New Generic.Dictionary(Of String, Generic.List(Of String))
    Dim verbose As Boolean = False
    Dim Summary As New Generic.Dictionary(Of String, EventLogSummary)

    Dim CommandLine As Arguments

    Private Function checkFilters(ByRef e As EventLogEntry) As Boolean
        Dim AndOp As Boolean = IIf(Not IsNothing(CommandLine("excOp")) AndAlso CommandLine("excOp").ToLower = "and", True, False)

        If excludes.Count > 0 Then
            For Each key As String In excludes.Keys
                For Each value As String In excludes.Item(key)
                    Select Case key.ToLower
                        Case "source"
                            If e.Source.ToLower.Contains(value) Then
                                If Not AndOp Then Return False
                            Else
                                If AndOp Then GoTo NextCondition
                            End If
                        Case "category"
                            If e.Category.ToLower.Contains(value) Then
                                If Not AndOp Then Return False
                            Else
                                If AndOp Then GoTo NextCondition
                            End If
                        Case "type"
                            If e.EntryType.ToString.ToLower.Contains(value) Then
                                If Not AndOp Then Return False
                            Else
                                If AndOp Then GoTo NextCondition
                            End If
                        Case "eventid"
                            If e.EventID.ToString.Contains(value) Then
                                If Not AndOp Then Return False
                            Else
                                If AndOp Then GoTo NextCondition
                            End If
                        Case "description"
                            If e.Message.ToLower.Contains(value) Then
                                If Not AndOp Then Return False
                            Else
                                If AndOp Then GoTo NextCondition
                            End If
                        Case Else
                    End Select
                Next
                If Not AndOp Then GoTo CheckIncludes
nextCondition:
            Next
        End If
CheckIncludes:
        AndOp = IIf(Not IsNothing(CommandLine("incOp")) AndAlso CommandLine("incOp").ToLower = "and", True, False)

        If includes.Count > 0 Then
            For Each key As String In includes.Keys
                For Each value As String In includes.Item(key)
                    Select Case key.ToLower
                        Case "source"
                            If e.Source.ToLower.Contains(value) Then
                                If Not AndOp Then Return True
                            Else
                                If AndOp Then Return False
                            End If
                        Case "category"
                            If e.Category.ToLower.Contains(value) Then
                                If Not AndOp Then Return True
                            Else
                                If AndOp Then Return False
                            End If
                        Case "type"
                            If e.EntryType.ToString.ToLower.Contains(value) Then
                                If Not AndOp Then Return True
                            Else
                                If AndOp Then Return False
                            End If
                        Case "eventid"
                            If e.EventID.ToString.Contains(value) Then
                                If Not AndOp Then Return True
                            Else
                                If AndOp Then Return False
                            End If
                        Case "description"
                            If e.Message.ToLower.Contains(value) Then
                                If Not AndOp Then Return True
                            Else
                                If AndOp Then Return False
                            End If
                        Case Else
                    End Select
                Next
            Next
            Return AndOp
        Else
            Return True
        End If
    End Function

    Private Function EscEntryType(ByRef e As EventLogEntry) As EventLogEntryType
        If Not IsNothing(CommandLine("C")) Then
            Dim AndOp As Boolean = IIf(Not IsNothing(CommandLine("cOp")) AndAlso CommandLine("cOp").ToLower = "and", True, False)
            Dim conditions As String() = CommandLine("C").ToLower.Split(",")
            For Each condition As String In conditions
                Dim keyValuePair As String() = condition.Split(":")
                Select Case keyValuePair(0).ToLower
                    Case "source"
                        If e.Source.ToLower.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Error
                        Else
                            If AndOp Then GoTo CheckWarning
                        End If
                    Case "category"
                        If e.Category.ToLower.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Error
                        Else
                            If AndOp Then GoTo CheckWarning
                        End If
                    Case "type"
                        If e.EntryType.ToString.ToLower.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Error
                        Else
                            If AndOp Then GoTo CheckWarning
                        End If
                    Case "eventid"
                        If e.EventID.ToString.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Error
                        Else
                            If AndOp Then GoTo CheckWarning
                        End If
                    Case "description"
                        If e.Message.ToLower.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Error
                        Else
                            If AndOp Then GoTo CheckWarning
                        End If
                    Case Else
                End Select
            Next
            If AndOp Then Return EventLogEntryType.Error
        End If
CheckWarning:
        If Not IsNothing(CommandLine("W")) Then
            Dim AndOp As Boolean = IIf(Not IsNothing(CommandLine("wOp")) AndAlso CommandLine("wOp").ToLower = "and", True, False)
            Dim conditions As String() = CommandLine("W").ToLower.Split(",")
            For Each condition As String In conditions
                Dim keyValuePair As String() = condition.Split(":")
                Select Case keyValuePair(0).ToLower
                    Case "source"
                        If e.Source.ToLower.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Warning
                        Else
                            If AndOp Then Exit For
                        End If
                    Case "category"
                        If e.Category.ToLower.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Warning
                        Else
                            If AndOp Then Exit For
                        End If
                    Case "type"
                        If e.EntryType.ToString.ToLower.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Warning
                        Else
                            If AndOp Then Exit For
                        End If
                    Case "eventid"
                        If e.EventID.ToString.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Warning
                        Else
                            If AndOp Then Exit For
                        End If
                    Case "description"
                        If e.Message.ToLower.Contains(keyValuePair(1)) Then
                            If Not AndOp Then Return EventLogEntryType.Warning
                        Else
                            If AndOp Then Exit For
                        End If
                    Case Else
                End Select
            Next
            If AndOp Then Return EventLogEntryType.Warning
        End If
        Return EventLogEntryType.Information
    End Function

    Sub Main(ByVal Args As String())
        startTime = Date.Now
        CommandLine = New Arguments(Args)
        If Not IsNothing(CommandLine("period")) Then
            If CommandLine("period").ToLower = "all" Then
                period = -1
            ElseIf IsNumeric(CommandLine("period")) Then
                period = CDbl(CommandLine("period"))
            End If
        End If

        If Not IsNothing(CommandLine("include")) Then
            Dim includeStrings = CommandLine("include").Split(",")
            For Each inc As String In includeStrings
                Dim tempValue As String() = inc.Split(":")
                If tempValue.Length <> 2 OrElse tempValue(0).Trim = "" OrElse tempValue(1).Trim = "" Then Continue For
                If includes.ContainsKey(tempValue(0)) Then
                    includes(tempValue(0)).Add(tempValue(1).ToLower)
                Else
                    Dim l As New List(Of String)
                    l.Add(tempValue(1).ToLower)
                    includes.Add(tempValue(0), l)
                End If
            Next
        End If

        If Not IsNothing(CommandLine("exclude")) Then
            Dim excludeStrings = CommandLine("exclude").Split(",")
            For Each exc As String In excludeStrings
                Dim tempValue As String() = exc.Split(":")
                If tempValue.Length <> 2 OrElse tempValue(0).Trim = "" OrElse tempValue(1).Trim = "" Then Continue For
                If excludes.ContainsKey(tempValue(0)) Then
                    excludes(tempValue(0)).Add(tempValue(1).ToLower)
                Else
                    Dim l As New List(Of String)
                    l.Add(tempValue(1).ToLower)
                    excludes.Add(tempValue(0), l)
                End If
            Next
        End If

        If Not IsNothing(CommandLine("v")) OrElse Not IsNothing(CommandLine("verbose")) Then verbose = True
        If Not IsNothing(CommandLine("logs")) Then logsToSearch = CommandLine("logs").ToLower.Split(",")
        If Not IsNothing(CommandLine("t")) AndAlso IsNumeric(CommandLine("t")) Then timeout = CommandLine("t")
        If Not IsNothing(CommandLine("Timeout")) AndAlso IsNumeric(CommandLine("Timeout")) Then timeout = CommandLine("Timeout")
        If verbose Then Console.WindowWidth = Console.LargestWindowWidth * 0.85
        If Not IsNothing(CommandLine("version")) Then
            Console.WriteLine("Nagios Event Log Plugin " & System.Reflection.Assembly.GetExecutingAssembly().GetName().Version().ToString)
            Exit Sub
        End If
        If Not IsNothing(CommandLine("h")) OrElse Not IsNothing(CommandLine("help")) Then
            Console.WriteLine("Nagios Event Log Plugin")
            Console.WriteLine("")
            Console.WriteLine("syntax EventLogPlugin.exe [-h] [-t timeout] [--logs {'logA,logB' || ALL}] [--verbose] [-V]")
            Console.WriteLine("       [--incOp {And || Or}] [--excOp {And || Or}]")
            Console.WriteLine("       [--include '{Source||Category||Type||EventID||Description}:keyword,...']")
            Console.WriteLine("       [--exclude '{Source||Category||Type||EventID||Description}:keyword,...']")
            Console.WriteLine("       [-wOp {And || Or}] [-cOp {And || Or}]")
            Console.WriteLine("       [-w  '{Source||Category||Type||EventID||Description}:keyword,...']")
            Console.WriteLine("       [-c  '{Source||Category||Type||EventID||Description}:keyword,...']")
            Console.WriteLine("       [-wMsg 'Error xyz'] [-cMsg 'Error abc'] [--period {period || ALL}]")
            Console.WriteLine("")
            Console.WriteLine("Valid exit codes are {0: OK, 1: Warning, 2:Critical, 3:Unknown}")
            Console.WriteLine("")
            Console.WriteLine("Parameters")
            Console.WriteLine("    --include params  Takes a comma separated list of (field:value) pairs to")
            Console.WriteLine("                      include in the results. If not specified will include all.")
            Console.WriteLine("    --incOp op        Specifies which logical operator to use when multiple")
            Console.WriteLine("                      include parameters are included. Default OR")
            Console.WriteLine("    --exclude params  Takes a comma separated list of (field:value) pairs to")
            Console.WriteLine("                      exclude from the results. If not specified will exclude nothing.")
            Console.Write("-- more --")
            Console.ReadKey()
            Console.WriteLine("")
            Console.WriteLine("    --excOp op        Specifies which logical operator to use when multiple")
            Console.WriteLine("                      exclue parameters are included. Default OR")
            Console.WriteLine("    --logs lognames   Takes a comma separated list of logs to include in the")
            Console.WriteLine("                      search as a paramter. Default is 'Application,Security,System'.")
            Console.WriteLine("                      Can specify ALL to search all logs.")
            Console.WriteLine("    --period mins     Only searches event log entries that occurred in the last")
            Console.WriteLine("                      specified amount of minutes. Default is 1440 (1 Day).")
            Console.WriteLine("                      Can specify ALL to search all events in logs.")
            Console.WriteLine(" -t --timeout mins    Plugin timeout in seconds. Default is 10.")
            Console.WriteLine("                      Plugin will halt execution after this time.")
            Console.WriteLine(" -v --verbose         Write additional information to standard output.")
            Console.WriteLine(" -w params            Takes a comma separated list of (field:value) pairs,")
            Console.WriteLine("                      matching entries get escalated to a warning.")
            Console.WriteLine(" -wOp op              Specifies which logical operator to use when multiple")
            Console.WriteLine("                      w parameters are included. Default OR")
            Console.WriteLine(" -wMsg message        Specifies the message to be displayed it the -w switch")
            Console.WriteLine("                      escalates an entry")
            Console.WriteLine(" -c params            Takes a comma separated list of (field:value) pairs")
            Console.WriteLine("                      matching entries get escalated to a critical.")
            Console.WriteLine(" -cOp op              Specifies which logical operator to use when multiple")
            Console.WriteLine("                      c parameters are included. Default OR")
            Console.Write("-- more --")
            Console.ReadKey()
            Console.WriteLine("")
            Console.WriteLine(" -cMsg message        Specifies the message to be displayed it the -c switch")
            Console.WriteLine("                      escalates an entry")
            Console.WriteLine(" -V --version         Displays version info.")
            Console.WriteLine(" -h --help            Displays this screen.")
            Exit Sub
        End If
        For Each log As System.Diagnostics.EventLog In EventLog.GetEventLogs()
            If Array.IndexOf(logsToSearch, "all") < 0 AndAlso Array.IndexOf(logsToSearch, log.Log.ToLower) < 0 Then
                Continue For
            End If
            If verbose Then Console.WriteLine("Searching: " & log.LogDisplayName)

            For entryCounter As Integer = log.Entries.Count - 1 To 0 Step -1
                If startTime.AddSeconds(timeout) <= Date.Now Then
                    timeout = -10
                    GoTo Timeout
                End If
                Dim entry As System.Diagnostics.EventLogEntry = log.Entries(entryCounter)
                If entry.EntryType = EventLogEntryType.FailureAudit OrElse entry.EntryType = EventLogEntryType.SuccessAudit Then Continue For
                If period < 0 OrElse entry.TimeGenerated >= Date.Now.AddMinutes(-period) Then
                    If Not checkFilters(entry) Then Continue For
                    Dim e As EventLogSummary
                    If Summary.ContainsKey(entry.Source) Then
                        e = Summary(entry.Source)
                    Else
                        e = New EventLogSummary
                        Summary.Add(entry.Source, e)
                    End If
                    Select Case EscEntryType(entry)
                        Case EventLogEntryType.Error
                            If e.message = "" Then e.message = IIf(Not IsNothing(CommandLine("cMsg")), CommandLine("cMsg"), "ESC ERROR")
                            FoundError = True
                        Case EventLogEntryType.Warning
                            If e.message = "" Then e.message = IIf(Not IsNothing(CommandLine("wMsg")), CommandLine("wMsg"), "ESC WARN")
                            FoundWarning = True
                    End Select

                    Select Case entry.EntryType
                        Case EventLogEntryType.Error
                            e.Errors += 1
                        Case EventLogEntryType.Warning
                            e.Warnings += 1
                        Case Else
                            e.Info += 1
                    End Select
                    If verbose Then
                        Console.Write(" " & entry.TimeGenerated.ToString())
                        For i As Integer = 0 To 22 - entry.TimeGenerated.ToString().Length
                            Console.Write(" ")
                        Next
                        Console.Write(IIf(entry.EntryType.ToString = "0", "None", entry.EntryType.ToString))
                        For i As Integer = 0 To 20 - IIf(entry.EntryType.ToString = "0", "None", entry.EntryType.ToString).Length
                            Console.Write(" ")
                        Next
                        Console.Write(IIf(entry.Category = "(0)", "None", entry.Category))
                        For i As Integer = 0 To 25 - IIf(entry.Category = "(0)", "None", entry.Category).Length
                            Console.Write(" ")
                        Next
                        Console.WriteLine(entry.Source)
                        'Console.WriteLine(entry.Message & vbCrLf & vbCrLf)
                    End If
                    If entry.EntryType = EventLogEntryType.Error Then FoundError = True
                    If entry.EntryType = EventLogEntryType.Warning OrElse entry.EntryType = EventLogEntryType.FailureAudit Then FoundWarning = True
                Else
                    entryCounter = 0
                End If
            Next
            If verbose Then Console.WriteLine("Done" & vbCrLf)
        Next
timeout:

        Dim sort As New Generic.SortedList(Of Integer, String)

        For Each key As String In Summary.Keys
            Dim s As EventLogSummary = Summary(key)
            Dim extra As Integer = 0
            While sort.ContainsKey(-(s.Errors * 1000 * IIf(s.message = "", 1, 100) + s.Warnings * 100 + s.Info * 10 + extra))
                extra += 1
            End While
            sort.Add(-(s.Errors * 1000 * IIf(s.message = "", 1, 100) + s.Warnings * 100 + s.Info * 10 + extra), key & "(" & s.Errors & ":" & s.Warnings & ":" & s.Info & IIf(s.message <> "", " " & s.message, "") & ")")
        Next


        Dim buffer As String = IIf(FoundError, "CRITICAL - ", IIf(FoundWarning, "WARNING - ", "OK - "))

        For Each Logevent As String In sort.Values
            buffer &= Logevent & ";"
            If buffer.Length >= 250 Then
                buffer = buffer.Substring(0, 247) & "..."
                Exit For
            End If
        Next
        Console.Write(buffer)
        If verbose Then Console.WriteLine(vbCrLf & vbCrLf & IIf(timeout <> -10, "Completed in ", "Timed out after ") & Date.Now.Subtract(startTime).TotalSeconds.ToString("N1") & " seconds.")
        Environment.ExitCode = IIf(FoundError, 2, IIf(FoundWarning, 1, 0))
        'Console.ReadKey()
    End Sub

End Module
